/**
 * Dash Filters Plugin for Booking.com
 *
 * Dependencies:
 *   - jQuery
 *
 * @author Volodymyr Sheremeta <volodymyr.sh@sisense.com>
 */
(function (ns) {
    'use strict';

    /**
     * DashFilters Class
     *
     * @constructor
     * @param {Object} params {dashboard, importantFilters}
     * @return {ns.DashFilters}
     */
    ns.DashFilters = function (params) {

        var
            dashboard = params.dashboard,
            importantFilters = _.map(params.importantFilters || [], function (n) {
                return n.toLowerCase();
            }),
            filtersWrapper = $('#prism-rightview .ew-panel'),
            buttonId = 'bk-filter-button-' + dashboard.oid,
            button = $(
                '<div id="' + buttonId + '" class="bk-filters">' +
                    '<div class="bk-line"></div>' +
                    '<div class="bk-text"></div>' +
                '</div>'
            ),

            // private methods
            init = function () {
                sortFilters();
                hideFilters();

                var insertedButton = $('#' + buttonId);

                // hack for proper rendering
                setTimeout(function () {
                    var
                        importantFiltersElements = filtersWrapper.find('.f-wrapper:not(.bk-filters-hide)'),
                        hiddenFiltersElements = filtersWrapper.find('.f-wrapper.bk-filters-hide')
                        ;

                    if (insertedButton.length && !insertedButton.hasClass('gray') || !insertedButton.length) {
                        hiddenFiltersElements.addClass('bk-filters-hidden');
                    }

                    if (!importantFiltersElements.length) {
                        hiddenFiltersElements.removeClass('bk-filters-hidden');
                    }
                    importantFiltersElements.last().after(insertedButton.length ? insertedButton : button);

                }, 25);

                // when button inserted - stop
                if (insertedButton.length) {
                    return;
                }

                button.off('click', clickHandler);
                button.on('click', clickHandler);
            },
            clickHandler = function () {

                if (button.hasClass('gray'))
                    filtersWrapper.find('.f-wrapper.bk-filters-hide').addClass('bk-filters-hidden');
                else
                    filtersWrapper.find('.f-wrapper.bk-filters-hide').removeClass('bk-filters-hidden');

                button.toggleClass('gray');
            },
            hideFilters = function () {
                var filtersTitles = [];

                dashboard.filters.$$items.forEach(function (filter) {

                    filter.getJAQL().forEach(function (jaql) {
                        if (~importantFilters.indexOf(jaql.dim.toLowerCase()))
                            filtersTitles.push(jaql.title.toLowerCase());
                    });

                });

                filtersWrapper.find('.f-wrapper').each(function (i, item) {
                    var
                        filter = $(item),
                        isCaption = false
                        ;

                    filter.find('span.ew-i-caption').each(function (i, caption) {
                        if (~filtersTitles.indexOf($(caption).text().toLowerCase()))
                            isCaption = true;
                    });

                    if (!isCaption)
                        filter.addClass('bk-filters-hide');
                });
            },
            sortFilters = function () {

                var filters = dashboard.filters.$$items;

                function getIdx(dim) {
                    var idx = -1;

                    for (var i = 0; i < filters.length; i++) {
                        var
                            item = filters[i],
                            relevant = _.some(item.getJAQL(), function (jaql) {								
								return window.sisenseUtils ? window.sisenseUtils.string.equalsIgnoreCase(jaql.dim, dim) : jaql.dim.equalsIgnoreCase(dim);
                            });

                        if (relevant) {
                            idx = i;
                            break;
                        }
                    }
                    return idx;
                }

                importantFilters.forEach(function (item) {
                    var
                        idx = getIdx(item),
                        taken = dashboard.filters.$$items[idx]
                        ;

                    if (!~idx) return;
                    // We have to do this because there is an override of splice
                    // in $$item that doesn't return the spliced items...
                    dashboard.filters.$$items.splice(idx, 1);
                    dashboard.filters.$$items.unshift(taken);
                });

            };

        init();//initializing plugin

        return this;
    };

    ns.DashFilters.prototype.constructor = ns.DashFilters;

}(window));